% Main program to perform BSS using algorithms PALS, STALS, and BCVMFB
% Main program allowing separation
% This program allows to process 1D or 2D data

clear
close all
clc

%% Necessary paths
% path to data --> to be downloaded from https://doi.org/10.5281/zenodo.4897295
addpath(genpath('data'));
% path to algorithms
addpath(genpath('algos'));
% path to tools
addpath(genpath('utils'));
% path to third-party functions (such as bss_eval)
addpath(genpath('third_party'));

%% Data loading
% we load mixing matrix A, sources S and mixtures X
% one can load '1D' or '2D' HSQC
pdata.dim='1D'; 
switch pdata.dim
    case '1D'
        load('Terpenes_US_1D.mat')
    case '2D'
        load('Terpenes_US_2D.mat')
    otherwise
        error('Data dimension unknown.');
end
% original spectra size
pdata.size = size(squeeze(X(1,:,:)));
% original data
pdata.Sorig=S;
pdata.Aorig=A;

%% Noise: one can estimate the variance in the mixtures
Xvar=squeeze(X(:,1:100,1));
sigma=std(detrend(Xvar(:)));
fprintf('Noise standard deviation estimated from X, sigma = %d \n',sigma);

%% Considered mixtures
pdata.type='simulated'; %real

switch pdata.type
    case 'real'
        % 1) We can consider real measured mixtures X
        % X is in the mat file --> nothing to do
        disp('Real case');
    case 'simulated'
        % 2) we can generate observations as X= AS + B (linear model)
        disp('Simulated case');
        pdata.seed=rng;
        X = genereObs(A,S,sigma);
    otherwise
        error('Data type for observations unknown');
end
% original data
pdata.Xorig=X;

%% Sizes
% Reshape to facilitate the AS product
S=reshape(S,size(S,1),size(S,2)*size(S,3));
X=reshape(X,size(X,1),size(X,2)*size(X,3));
%M*K = size of A;
%K*L = size of S
[M,K]=size(A);
L=size(S,2);

%% check negativity
% mixtures can have negative coefficients ... we can either project them or
% not
pdata.set2zero_flag = false;

if sum(sum(X<0))~=0
    fprintf('Warning: there are %d negative values...\n',sum(sum(X<0)));
    if pdata.set2zero_flag == true
        disp('negative values are set to zero');
        X(X<0)=0;
    else
        disp('No action taken for negative values');
    end
end

% Computation of the SNR (noisy case)
if sigma~=0
    snr = 10*log10(mean(mean((A*S).^2))/mean(mean((A*S-X).^2)));
    fprintf('The initial SNR is equal to %f dB\n',snr);
end

%% Algorithm parameters

%Fix the iteration number
palgo.niter=1000;

% Fix the regularizations on A and S
% regularizations are of the form alpha*|.|^p
palgo.alpha_A=0.0;
palgo.alpha_S=0.01*sigma;

palgo.p_A=1;
palgo.p_S=1;

% addition of a non negativity constraint [eta_inf,eta_sup]
palgo.eta_inf = eps;
palgo.eta_sup = inf;


%% Initialization with Jade
% we save the original A and S
pdata.S=S;
pdata.A=A;

[pdata.Ai,pdata.Si]= Init_Separation(X,K);

clear A S

% option of data normalization
% as separation is performed up to a permutation and a normalization, one
% can choose to normalize data at the beginning or at the end of the
% process (can also be tested at each iteration but with no convergence
% guarantee)
pdata.Norm.FlagInit=true;
pdata.Norm.FlagIter=false;
pdata.Norm.FlagEnd=false;
% we can either normalize the columns of A (be careful, these are rows of A
% that could be sum up to 1) or the rows of S (each source)
% normaliaation type lp=sum |.|^p -->  l^1 (1) or l^2_2 (2)
pdata.Norm.p=1;
% we normalize along the columns (1) (the norm of the columns equal to 1)
% or the rows (2) (the norm of the rows equal to 1)
pdata.Norm.dim=1;
% we normalise A or S
pdata.Norm.x='A';

if pdata.Norm.FlagInit
    [pdata.Ai, pdata.Si] = normalise(pdata.Ai, pdata.Si,pdata);
end
% NB: in ltfat one function exists http://ltfat.github.io/doc/sigproc/normalize.html


% one can choose a stopping criteria (A no longer varies, S no longer
% varies, and the criterion non longer varies) in addition to the maximum
% number of iterations
palgo.stop.DifA = false;
palgo.stop.DifS = false;
palgo.stop.ObjFct = true;

% we fix the threshold for the stopping criterion
palgo.stop.seuil=eps;

% we fix the preconditionning technique for BCVMFB
% 'Lips' for Lipschitz constant (no preconditionning = PALM)
% 'LS' for Lee and Seung method
palgo.precond = 'LS';

% algorithm stepsize
palgo.gamma = 0.99;

% algorithm
palgo.algo='BCVMFB';

switch palgo.algo
    
    case 'BCVMFB'
        %% BC-VMFB algorithm (alternated approach on A and S)
        % Forward-Backward algorithm with or without preconditionning
        tic
        pres=algo_BCVMFB(X,pdata,palgo);
        toc
        pres.algname='BCVMFB';
        
    case 'PALS'
        %% PALS algorithm (alternated approach on A and S)
        % Projected least square >0
        tic
        pres=algo_PALS(X,pdata,palgo);
        toc
        pres.algname='PALS';
        
    case 'STALS'
        %% STALS algorithm (alternated approach on A and S)
        % Soft thresholded projected least squares >0 (sparsity on S)
        tic
        pres=algo_STALS(X,pdata,palgo);
        toc
        pres.algname='STALS';
        
    case 'LSfro'
        %% LSfro algorithm (alternated approach on A and S)
        % Lee and Seung algorithm with Frobenius norm
        tic
        pres=algo_LSfro(X,pdata,palgo);
        toc
        pres.algname='LSfro';
        
    case 'LSkl'
        %% LSkl algorithm (alternated approach on A and S)
        % Lee and Seung algorithm with Kullback-Leibler divergence
        tic
        pres=algo_LSkl(X,pdata,palgo);
        toc
        pres.algname='LSkl';
        
end

if pdata.Norm.FlagEnd
    [pres.A, pres.S] = normalise(pres.A,pres.S,pdata);
end

%% Final SNR computation
if sigma~=0
    snr = 10*log10(mean(mean((pdata.A*pdata.S).^2))/mean(mean((pdata.A*pdata.S-pres.A*pres.S).^2)));
    fprintf('The final SNR is equal to %f dB\n',snr);
end

%% Plots for the results

% Plots and dipslay of the SxR and Amari index
PlotBSSResults(pres,'Terpenes');
pres_summary = SummarizeBSSResults(pres,'Terpenes');
DisplayBSSSummary(pres_summary);

% Plots of the sources S along with ground truth
% Be careful to source ordering
[pres.S,pres.A, ~] = reorderAS(pdata.S,pres.S,pres.A);

switch pdata.dim
    case '1D'
        %% Plots of the mixture spectra and the sources
        figure
        plot(xVec,pdata.S')
        legend('S1','S2','S3','S4')
        xlabel(['^1 H (' xUnits ')'])
        set(gca,'xdir','reverse')
        axis tight
        title('^1H NMR original pure compounds spectra')
        figure
        plot(xVec,pres.S')
        legend('S1','S2','S3','S4')
        xlabel(['^1 H (' xUnits ')'])
        set(gca,'xdir','reverse')
        axis tight
        title('^1H NMR estimated pure compounds spectra')
    case {'2D','2DNUS'}
        %% Plots of the mixture spectra and the sources
        % May take several minutes
        sig_name={'S1','S2','S3','S4'};
        fig_name='2D ^1H-^{13}C HSQC original pure compound spectra';
        plot2D_contour(pdata.S,xVec,yVec,sig_name,fig_name)
        sig_name={'S1','S2','S3','S4'};
        fig_name='2D ^1H-^{13}C HSQC estimated pure compound spectra';
        plot2D_contour(pres.S,xVec,yVec,sig_name,fig_name)
    otherwise
        error('Data dimension unknown for plot.');
end